@@ -1,22 +1,22 @@
 /* pinentry-curses.c - A secure curses dialog for PIN entry, library version
-   Copyright (C) 2002 g10 Code GmbH
-   
-   This file is part of PINENTRY.
-   
-   PINENTRY is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
- 
-   PINENTRY is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
- 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA  */
+ * Copyright (C) 2002, 2015 g10 Code GmbH
+ *
+ * This file is part of PINENTRY.
+ *
+ * PINENTRY is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PINENTRY is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0+
+ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -46,8 +46,16 @@
 #include <wchar.h>
 #endif /*HAVE_WCHAR_H*/
 
+#include <assuan.h>
+
 #include "pinentry.h"
 
+#if GPG_ERROR_VERSION_NUMBER < 0x011900 /* 1.25 */
+# define GPG_ERR_WINDOW_TOO_SMALL 301
+# define GPG_ERR_MISSING_ENVVAR   303
+#endif
+
+
 /* FIXME: We should allow configuration of these button labels and in
    any case use the default_ok, default_cancel values if available.
    However, I have no clue about curses and localization.  */
@@ -83,7 +91,6 @@ struct dialog
   int pin_size;
   /* Cursor location in PIN field.  */
   int pin_loc;
-  char *pin;
   int pin_max;
   /* Length of PIN.  */
   int pin_len;
@@ -97,6 +104,8 @@ struct dialog
   int notok_y;
   int notok_x;
   char *notok;
+
+  pinentry_t pinentry;
 };
 typedef struct dialog *dialog_t;
 
@@ -126,7 +135,7 @@ typedef char CH;
    there is a forced line break.  A full line is returned and will be
    continued in the next line.  */
 static void
-collect_line (int maxwidth, wchar_t **start_p, int *len_p)
+collect_line (int maxwidth, CH **start_p, int *len_p)
 {
   int last_space = 0;
   int len = *len_p;
@@ -199,6 +208,8 @@ utf8_to_local (char *lc_ctype, char *string)
   memset (&ps, 0, sizeof(mbstate_t));
   mbsrtowcs (wcs, &p, len, &ps);
 
+  free (local);
+
  leave:
   if (old_ctype)
     {
@@ -232,6 +243,8 @@ dialog_create (pinentry_t pinentry, dialog_t dialog)
   CH *error = NULL;
   CH *prompt = NULL;
 
+  dialog->pinentry = pinentry;
+
 #define COPY_OUT(what)							\
   do									\
     if (pinentry->what)							\
@@ -240,38 +253,67 @@ dialog_create (pinentry_t pinentry, dialog_t dialog)
         if (!what)							\
 	  {								\
 	    err = 1;							\
+            pinentry->specific_err = gpg_error (GPG_ERR_LOCALE_PROBLEM); \
+            pinentry->specific_err_loc = "dialog_create_copy";          \
 	    goto out;							\
 	  }								\
       }									\
   while (0)
-    
+
   COPY_OUT (description);
   COPY_OUT (error);
   COPY_OUT (prompt);
 
+  /* There is no pinentry->default_notok.  Map it to
+     pinentry->notok.  */
+#define default_notok notok
 #define MAKE_BUTTON(which,default)					\
   do									\
     {									\
       char *new = NULL;							\
-      if (pinentry->which)						\
+      if (pinentry->default_##which || pinentry->which)			\
         {								\
-          int len = strlen (pinentry->which);				\
+	  int len;							\
+	  char *msg;							\
+	  int i, j;							\
+									\
+	  msg = pinentry->which;					\
+	  if (! msg)							\
+	    msg = pinentry->default_##which;				\
+          len = strlen (msg);						\
+									\
           new = malloc (len + 3);				       	\
 	  if (!new)							\
 	    {								\
 	      err = 1;							\
+              pinentry->specific_err = gpg_error_from_syserror ();	\
+              pinentry->specific_err_loc = "dialog_create_mk_button";   \
 	      goto out;							\
 	    }								\
-	  new[0] = '<';							\
-	  memcpy (&new[1], pinentry->which, len);			\
-          new[len + 1] = '>';						\
-	  new[len + 2] = '\0';						\
+									\
+	  new[0] = '<'; 						\
+	  for (i = 0, j = 1; i < len; i ++, j ++)			\
+	    {								\
+	      if (msg[i] == '_')					\
+		{							\
+		  i ++;							\
+		  if (msg[i] == 0)					\
+		    /* _ at end of string.  */				\
+		    break;						\
+		}							\
+	      new[j] = msg[i];						\
+	    }								\
+									\
+	  new[j] = '>';							\
+	  new[j + 1] = 0;						\
         }								\
       dialog->which = pinentry_utf8_to_local (pinentry->lc_ctype,	\
 					      new ? new : default);	\
       if (!dialog->which)						\
         {								\
 	  err = 1;							\
+          pinentry->specific_err = gpg_error (GPG_ERR_LOCALE_PROBLEM);	\
+          pinentry->specific_err_loc = "dialog_create_utf8conv";        \
 	  goto out;							\
 	}								\
     }									\
@@ -306,7 +348,7 @@ dialog_create (pinentry_t pinentry, dialog_t dialog)
       while (start[len - 1]);
       y++;
     }
-      
+
   if (pinentry->pin)
     {
       if (error)
@@ -333,10 +375,13 @@ dialog_create (pinentry_t pinentry, dialog_t dialog)
       y += 2;		/* Pin entry field.  */
     }
   y += 2;		/* OK/Cancel and bottom frame.  */
-  
+
   if (y > size_y)
     {
       err = 1;
+      pinentry->specific_err = gpg_error (size_y < 0? GPG_ERR_MISSING_ENVVAR
+                                          /* */     : GPG_ERR_WINDOW_TOO_SMALL);
+      pinentry->specific_err_loc = "dialog_create";
       goto out;
     }
 
@@ -391,11 +436,13 @@ dialog_create (pinentry_t pinentry, dialog_t dialog)
   if (x > size_x)
     {
       err = 1;
+      pinentry->specific_err = gpg_error (size_x < 0? GPG_ERR_MISSING_ENVVAR
+                                          /* */     : GPG_ERR_WINDOW_TOO_SMALL);
+      pinentry->specific_err_loc = "dialog_create";
       goto out;
     }
 
   dialog->pos = DIALOG_POS_NONE;
-  dialog->pin = pinentry->pin;
   dialog->pin_max = pinentry->pin_len;
   dialog->pin_loc = 0;
   dialog->pin_len = 0;
@@ -523,7 +570,7 @@ dialog_create (pinentry_t pinentry, dialog_t dialog)
       move (dialog->ok_y, dialog->ok_x);
       addstr (dialog->ok);
 
-      if (dialog->notok)
+      if (! pinentry->pin && dialog->notok)
 	{
 	  dialog->notok_y = ypos;
 	  /* Calculating the left edge of the middle button, rounding up.  */
@@ -654,15 +701,24 @@ dialog_switch_pos (dialog_t diag, dialog_pos_t new_pos)
 
 /* XXX Assume that field width is at least > 5.  */
 static void
-dialog_input (dialog_t diag, int chr)
+dialog_input (dialog_t diag, int alt, int chr)
 {
   int old_loc = diag->pin_loc;
-  assert (diag->pin);
+  assert (diag->pinentry->pin);
   assert (diag->pos == DIALOG_POS_PIN);
 
+  if (alt && chr == KEY_BACKSPACE)
+    /* Remap alt-backspace to control-W.  */
+    chr = 'w' - 'a' + 1;
+
   switch (chr)
     {
     case KEY_BACKSPACE:
+      /* control-h.  */
+    case 'h' - 'a' + 1:
+      /* ASCII DEL.  What Mac OS X apparently emits when the "delete"
+	 (backspace) key is pressed.  */
+    case 127:
       if (diag->pin_len > 0)
 	{
 	  diag->pin_len--;
@@ -676,10 +732,63 @@ dialog_input (dialog_t diag, int chr)
 	}
       break;
 
+    case 'l' - 'a' + 1: /* control-l */
+      /* Refresh the screen.  */
+      endwin ();
+      refresh ();
+      break;
+
+    case 'u' - 'a' + 1: /* control-u */
+      /* Erase the whole line.  */
+      if (diag->pin_len > 0)
+	{
+	  diag->pin_len = 0;
+	  diag->pin_loc = 0;
+	}
+      break;
+
+    case 'w' - 'a' + 1: /* control-w.  */
+      while (diag->pin_len > 0
+	     && diag->pinentry->pin[diag->pin_len - 1] == ' ')
+	{
+	  diag->pin_len --;
+	  diag->pin_loc --;
+	  if (diag->pin_loc < 0)
+	    {
+	      diag->pin_loc += diag->pin_size;
+	      if (diag->pin_loc > diag->pin_len)
+		diag->pin_loc = diag->pin_len;
+	    }
+	}
+      while (diag->pin_len > 0
+	     && diag->pinentry->pin[diag->pin_len - 1] != ' ')
+	{
+	  diag->pin_len --;
+	  diag->pin_loc --;
+	  if (diag->pin_loc < 0)
+	    {
+	      diag->pin_loc += diag->pin_size;
+	      if (diag->pin_loc > diag->pin_len)
+		diag->pin_loc = diag->pin_len;
+	    }
+	}
+
+      break;
+
     default:
       if (chr > 0 && chr < 256 && diag->pin_len < diag->pin_max)
 	{
-	  diag->pin[diag->pin_len] = (char) chr;
+	  /* Make sure there is enough room for this character and a
+	     following NUL byte.  */
+	  if (! pinentry_setbufferlen (diag->pinentry, diag->pin_len + 2))
+	    {
+	      /* Bail.  Here we use a simple approach.  It would be
+                 better to have a pinentry_bug function.  */
+              assert (!"setbufferlen failed");
+              abort ();
+	    }
+
+	  diag->pinentry->pin[diag->pin_len] = (char) chr;
 	  diag->pin_len++;
 	  diag->pin_loc++;
 	  if (diag->pin_loc == diag->pin_size && diag->pin_len < diag->pin_max)
@@ -710,15 +819,18 @@ dialog_input (dialog_t diag, int chr)
 static int
 dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
 {
+  int confirm_mode = !pinentry->pin;
   struct dialog diag;
   FILE *ttyfi = NULL;
   FILE *ttyfo = NULL;
   SCREEN *screen = 0;
   int done = 0;
   char *pin_utf8;
+  int alt = 0;
 #ifndef HAVE_DOSISH_SYSTEM
   int no_input = 1;
 #endif
+
 #ifdef HAVE_NCURSESW
   char *old_ctype = NULL;
 
@@ -736,14 +848,20 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
     {
       ttyfi = fopen (tty_name, "r");
       if (!ttyfi)
-	return -1;
+        {
+          pinentry->specific_err = gpg_error_from_syserror ();
+          pinentry->specific_err_loc = "open_tty_for_read";
+          return confirm_mode? 0 : -1;
+        }
       ttyfo = fopen (tty_name, "w");
       if (!ttyfo)
 	{
 	  int err = errno;
 	  fclose (ttyfi);
 	  errno = err;
-	  return -1;
+          pinentry->specific_err = gpg_error_from_syserror ();
+          pinentry->specific_err_loc = "open_tty_for_write";
+	  return confirm_mode? 0 : -1;
 	}
       screen = newterm (tty_type, ttyfo, ttyfi);
       set_term (screen);
@@ -755,7 +873,9 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
           if (!(isatty(fileno(stdin)) && isatty(fileno(stdout))))
             {
               errno = ENOTTY;
-              return -1;
+              pinentry->specific_err = gpg_error_from_syserror ();
+              pinentry->specific_err_loc = "isatty";
+              return confirm_mode? 0 : -1;
             }
 	  init_screen = 1;
 	  initscr ();
@@ -763,16 +883,26 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
       else
 	clear ();
     }
-  
+
   keypad (stdscr, TRUE); /* Enable keyboard mapping.  */
   nonl ();		/* Tell curses not to do NL->CR/NL on output.  */
   cbreak ();		/* Take input chars one at a time, no wait for \n.  */
   noecho ();		/* Don't echo input - in color.  */
 
+  if (pinentry->ttyalert)
+    {
+      if (! strcmp(pinentry->ttyalert, "beep"))
+	beep ();
+      else if (! strcmp(pinentry->ttyalert, "flash"))
+	flash ();
+    }
+
   if (has_colors ())
     {
       start_color ();
+#ifdef NCURSES_VERSION
       use_default_colors ();
+#endif
 
       if (pinentry->color_so == PINENTRY_COLOR_DEFAULT)
 	{
@@ -792,10 +922,28 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
     }
   refresh ();
 
-  /* XXX */
+  /* Create the dialog.  */
   if (dialog_create (pinentry, &diag))
-    return -2;
-  dialog_switch_pos (&diag, diag.pin ? DIALOG_POS_PIN : DIALOG_POS_OK);
+    {
+      /* Note: pinentry->specific_err has already been set.  */
+      endwin ();
+      if (screen)
+        delscreen (screen);
+
+#ifdef HAVE_NCURSESW
+      if (old_ctype)
+        {
+          setlocale (LC_CTYPE, old_ctype);
+          free (old_ctype);
+        }
+#endif
+      if (ttyfi)
+        fclose (ttyfi);
+      if (ttyfo)
+        fclose (ttyfo);
+      return -2;
+    }
+  dialog_switch_pos (&diag, confirm_mode? DIALOG_POS_OK : DIALOG_POS_PIN);
 
 #ifndef HAVE_DOSISH_SYSTEM
   wtimeout (stdscr, 70);
@@ -810,22 +958,32 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
       if (timed_out && no_input)
 	{
 	  done = -2;
+          pinentry->specific_err = gpg_error (GPG_ERR_TIMEOUT);
 	  break;
 	}
 #endif
 
       switch (c)
 	{
-#ifndef HAVE_DOSISH_SYSTEM
 	case ERR:
+#ifndef HAVE_DOSISH_SYSTEM
 	  continue;
+#else
+          done = -2;
+          break;
 #endif
+
+	case 27: /* Alt was pressed.  */
+	  alt = 1;
+	  /* Get the next key press.  */
+	  continue;
+
 	case KEY_LEFT:
 	case KEY_UP:
 	  switch (diag.pos)
 	    {
 	    case DIALOG_POS_OK:
-	      if (diag.pin)
+	      if (!confirm_mode)
 		dialog_switch_pos (&diag, DIALOG_POS_PIN);
 	      break;
 	    case DIALOG_POS_NOTOK:
@@ -873,22 +1031,22 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
 	      if (diag.notok)
 		dialog_switch_pos (&diag, DIALOG_POS_NOTOK);
 	      else
-		dialog_switch_pos (&diag, DIALOG_POS_CANCEL);	      
+		dialog_switch_pos (&diag, DIALOG_POS_CANCEL);
 	      break;
 	    case DIALOG_POS_NOTOK:
 	      dialog_switch_pos (&diag, DIALOG_POS_CANCEL);
 	      break;
 	    case DIALOG_POS_CANCEL:
-	      if (diag.pin)
-		dialog_switch_pos (&diag, DIALOG_POS_PIN);
-	      else
+	      if (confirm_mode)
 		dialog_switch_pos (&diag, DIALOG_POS_OK);
+	      else
+		dialog_switch_pos (&diag, DIALOG_POS_PIN);
 	      break;
 	    default:
 	      break;
 	    }
 	  break;
-  
+
 	case '\005':
 	  done = -2;
 	  break;
@@ -913,14 +1071,23 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
 
 	default:
 	  if (diag.pos == DIALOG_POS_PIN)
-	    dialog_input (&diag, c);
+	    dialog_input (&diag, alt, c);
 	}
 #ifndef HAVE_DOSISH_SYSTEM
       no_input = 0;
 #endif
+      if (c != -1)
+	alt = 0;
     }
   while (!done);
 
+  if (!confirm_mode)
+    {
+      /* NUL terminate the passphrase.  dialog_run makes sure there is
+         enough space for the terminating NUL byte.  */
+      diag.pinentry->pin[diag.pin_len] = 0;
+    }
+
   set_cursor_state (1);
   endwin ();
   if (screen)
@@ -944,7 +1111,7 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
   if (diag.notok)
     free (diag.notok);
 
-  if (pinentry->pin)
+  if (!confirm_mode)
     {
       pinentry->locale_err = 1;
       pin_utf8 = pinentry_local_to_utf8 (pinentry->lc_ctype, pinentry->pin, 1);
@@ -961,7 +1128,11 @@ dialog_run (pinentry_t pinentry, const char *tty_name, const char *tty_type)
   if (done == -2)
     pinentry->canceled = 1;
 
-  return diag.pin ? (done < 0 ? -1 : diag.pin_len) : (done < 0 ? 0 : 1);
+  /* In confirm mode return cancel instead of error.  */
+  if (confirm_mode)
+    return done < 0 ? 0 : 1;
+
+  return done < 0 ? -1 : diag.pin_len;
 }
 
 
